# Relace

Mějme dvě množiny $X, Y$ a představme si, že každý prvek $x \in X$ může (a nemusí) být  ve vztahu $R$ s libovolným počtem prvků $y \in Y$. Na tento vztah nejsou kladeny žádné další podmínky.
- tento vztah lze popsat jako výčet všech dvojic $(x, y)$ prvků, které spolu jsou ve vztahu $R$

### Definice

Jelikož je kartézský součin definován jako **množina** všech **uspořádaných dvojic** s prvním prvkem z množiny $X$ a druhým z množiny $Y$, definujeme relaci takto:

Relace z množiny $X$ do množiny $Y$ je libovolná podmnožina $R$ kartézského součinu $X \times Y$.

Jedná se o relaci **binární**, jelikož určuje vztah mezi dvěma prvky.
- definici můžeme zobecnit na **n-ární relace** (vztahy mezi n-ticemi prvků)

### Obory

- **levý**: $L(R) = \{ x \in X: \text{existuje nějaké } y \in Y \text{ tak, že } x \, R \, y \}$
	- všechny prvky $X$, které jsou v relaci s nějakým prvkem $Y$
- **pravý**: $P(R) = \{ y \in Y: \text{existuje nějaké } x \in X \text{ tak, že } x \, R \, y \}$
	- všechny prvky $Y$, které jsou v relaci s nějakým prvkem $X$

### Znázornění relací

- pomocí grafu
	- množina $X$ vlevo, množina $Y$ vpravo
	- dva body jsou spojeny čarou, pokud jsou prvky v relaci R
- pomocí kartézského součinu
	- matice $X \times Y$, kde na horizontální ose jsou prvky $X$ a na vertikální ose $Y$
	- body jsou vybarveny pouze na místech matice, kde jsou prvky v relaci R

### Skládání relací

**Definice**
- Nechť $R$ je relace z množiny $X$ do množiny $Y$ a $S$ je relace z množiny $Y$ do množiny $Z$. Pak **složení relací** $R$ a $S$ je relace $R \circ S \subset X \times Z$ z množiny $X$ do množiny $Z$, definováno takto
- $(x, z) \in R \circ S$, právě když existuje $y \in Y$ tak, že $x \, R \, y$ a $y \, S \, z$,
- kde $x \in X$ a $z \in Z$.

Skládání relací je možné pouze, pokud relace $R$ končí ve množině, kde $S$ začíná.

Věta o asociativitě skládání relací
- Nechť $R \subset X \times Y, S \subset Y \times Z$ a $T \subset Z \times W$ jsou relace. Potom $R \circ (S \circ T) = (R \circ S) \circ T$.

### Inverzní relace

**Definice**
- Relace **inverzní** k relaci $R \subset X \times Y$ je relace $R^{-1} \subset Y \times X$, definovaná vztahem
- $y \, R^{-1} \, x$ právě když $x \, R \, y$ pro $x \in X, y \in Y$.

# Zobrazení, funkce

**Zobrazení** (nebo také **funkce**) množiny $X$ do množiny $Y$ je **relace** $f \subset X \times Y$, pro kterou platí, že **pro každý prvek** $x \in X$ **existuje právě jeden prvek** $y \in Y$ tak, že $(x, y) \in f$. Skutečnost, že $f$ je zobrazením $X$ do $Y$, zapisujeme jako $f: X \to Y$.

## Druhy zobrazení

Zobrazení $f: X \to Y$ je
- **prosté** (injekce), pokud každé $y \in Y$ má nejvýše jeden vzor při zobrazení $f$,
- **na** (surjekce), pokud každé $y \in Y$ má alespoň jeden vzor při zobrazení $f$,
- **vzájemně jednoznačné** (bijekce), pokud je **prosté** a **na**, tedy
	- každé $y \in Y$ má právě jeden vzor při zobrazení $f$.

## Skládání zobrazení

Máme zobrazení $f : X \to Y, g : Y \to Z$, výsledná relace $f \circ g$ je zobrazení $X$ do $Z$, pro jehož hodnoty platí $(f \circ g)(x) = g(f(x))$.