# Hodnost matice

Počet nenulových řádků matice po provedení GEM (Gaussova eliminační metoda), kterou dostaneme matici s nenulovými čísly nad diagonálou a na ní.

Při použítí GEM můžeme:
- přičíst libovolný nenulový násobek řádku k jinému řádku,
- libovolně násobit jednotlivé řádky (ne nulou).

# Soustava rovnic

Soustava $m$ rovnic pro $n$ neznámých:

$$
\begin{matrix}
a_{11}x_{1} + a_{12}x_{2} + a_{13}x_{3} + \dots + a_{1n}x_{n} = b_{1} \\
a_{21}x_{1} + a_{22}x_{2} + a_{23}x_{3} + \dots + a_{2n}x_{n} = b_{2} \\
\vdots \qquad\qquad\qquad \vdots \\
a_{m1}x_{1} + a_{m2}x_{2} + a_{m3}x_{3} + \dots + a_{mn}x_{n} = b_{n}
\end{matrix}
$$

Soustavu zapíšeme maticově:

$$
A = \begin{bmatrix}
a_{11} & a_{12} & \dots & a_{1n} \\
a_{21} & a_{22} & \dots & a_{2n} \\
\vdots & \vdots & \vdots & \vdots \\
a_{m1} & a_{m2} & \dots & a_{mn}
\end{bmatrix}, \qquad \vec{x} = \begin{bmatrix}
x_{1} \\
x_{2} \\
\vdots \\
x_{n}
\end{bmatrix}, \qquad \vec{b} = \begin{bmatrix}
b_{1} \\
b_{2} \\
\vdots \\
b_{m}
\end{bmatrix}
$$

Potom A je **matice soustavy** (typu $m/n$), $\vec{x}$ je **vektor (sloupec) neznámých** a $\vec{b}$ je **vektor (sloupec) pravých stran**.

Soustavu zapisujeme jako $A\vec{x} = \vec{b}$.

Dvě soustavy se nazývají **ekvivalentní**, jestliže mají stejnou množinu řešení.

### Rozšířená matice soustavy

Zápis soustavy do matice, kde svislá čára značí $=$, značíme ji jako $A^R = [A \mid \vec{b}]$.

### Frobeniova podmínka řešitelnosti

Nehomogenní soustava rovnic $A\vec{x} = \vec{b}$ má řešení právě tehdy, když $hod(A^R) = hod(A)$.

### Typy soustav

- **homogenní**
	- s nulovým sloupcem vpravo (nemusí se psát)
- **nehomogenní**
	- s čísly vpravo oddělenými svislou čárou (značí $=$)

### Řešení soustavy

1. přepíšu do matice a vyřeším pomocí GEM/GJEM
2. najdu pivoty (první nenulové číslo v řádku) a ke sloupcům bez pivota přiřadím parametry (např.: $x_3 = t, t \in R$)
3. řádky zapíšu jako rovnice (např.: $2x_1 + 3x_2 + x_4 = 0$)
4. z rovnic vyjádřím jednotlivá x

#### Možná řešení

- soustava nemá řešení
- soustava má jedno řešení
- soustava má nekonečně mnoho řešení

# Eukleidův algoritmus

Algoritmus pro nalezení největšího společného dělitele čísel $a$ a $b$. Největšího společného dělitele označíme jako $\gcd(a, b)$.

Mějme $a = 57, b = 27$. Platí, že $c = \gcd(a, b)$ dělí $a$ a rovněž $b$, dělí tedy i rozdíl $a-b = 30$. Pokud nyní najdeme $\gcd(30, 27)$, získáme i $\gcd(57, 27)$. Aplikacím tohoto postupu dostaneme postupně $\gcd(30, 27) = \gcd(3, 27) = 3$, tedy i $\gcd(57, 27) = 3$.

TODO: Rozšířený euklidův algoritmus

# Stanovení inverzního prvku

Prvek $a^{-1}$, pro který platí $a \oplus a^{-1} = 0$.

**Příklad**: Pokud máme modulo 5, tak inverzní prvek k prvku $4$ bude $1$, protože $4 + 1 = 0$.